<?php

/**
 * @package Faslet
 * @version 5.2.5
 */
/*
Plugin Name: Faslet Size Me Up
Plugin URI: https://docs.faslet.net/docs/widget/woocommerce
Description: Faslet's Size Me Up widget for WooCommerce is a paid service that allows your customers to find their perfect fit on the first try. In order to activate it for your shop or to get information on the above values or arrange an integration call, please contact a Faslet Representative at info@faslet.me
Author: Faslet Team
Version: 5.2.5
Author URI: https://faslet.me
License: MIT
License URI: https://mit-license.org/
*/

$FASLET_version = '5.2.5';
$FASLET_debug_enabled = FALSE;
$FASLET_default_size_attr = 'pa_size';
$FASLET_default_color_attr = 'pa_color';
$FASLET_default_hook = 'woocommerce_before_add_to_cart_form';

function faslet_log($message, $force = FALSE)
{
	global $FASLET_debug_enabled;
	if ($FASLET_debug_enabled || $force) {
		print("<script>console.log(`Faslet: $message`)</script>");
	}
}

function faslet_size_me_up()
{
    global $FASLET_default_size_attr, $FASLET_default_color_attr, $FASLET_version;

    faslet_log("Woocommerce plugin: $FASLET_version", TRUE);

	$product_id = get_queried_object_id();
	$product = wc_get_product($product_id);

	$name = addslashes($product->get_name());
	$dom = new DomDocument();
	@$dom->loadHTML($product->get_image());
	$imgElt = $dom->getElementsByTagName('img');
	if (count($imgElt) > 0) {
		$imageUrl = $imgElt[0]->getAttribute('src');
	}

	$fallback_shop = $product->meta_exists("Faslet_Shop") ? $product->get_meta("Faslet_Shop") : "";
	$shop = get_option("Faslet_Shop", $fallback_shop);
	$faslet_product_identifier_over_tag = get_option("Faslet_Use_ProductId", FALSE);


    $brand = $product->meta_exists("Faslet_Brand") ? $product->get_meta("Faslet_Brand") : '';

    if(!isset($brand) || $brand === '') {
        $brand = $product->meta_exists("product_brand") ? $product->get_meta("product_brand") : "";
        faslet_log("Brand fallback plugin: $brand");
    }

	// TODO: This should be better, but we don't want to introduce clunky UX for partners, so for now this will do until we have something more concrete.
	if(!isset($brand) || $brand === '') {
    	$brand = $product->get_attribute("pa_merk");
    	faslet_log("Brand fallback attr nl: $brand");
	}

	if(!isset($brand) || $brand === '') {
        $brand = $product->get_attribute("pa_brand");
        faslet_log("Brand fallback attr en: $brand");
    }

    // Hack just for boutique-brands. They use tags to identify products, so let's have single tags as a worst-case fallback.
    if(!isset($brand) || $brand === '') {
        $productTags = get_the_terms( $product_id, 'product_tag' );

        if(is_array($productTags)) {
            faslet_log('Tags array found');

            $tagCount = count($productTags);

            if($tagCount == 1) {
                $tag_title = $productTags[0]->name; // tag name
                $brand = $tag_title;
                faslet_log("Brand fallback tags: $brand");
            }
        } else {
            faslet_log('Tags not an array');
        }
    }


    $tag = $product->meta_exists("Faslet_Tag") ? $product->get_meta("Faslet_Tag") : "";
	$faslet_product_id = $product->meta_exists("Faslet_ProductId") ? $product->get_meta("Faslet_ProductId") : $product_id;

	faslet_log("Product details $shop, $brand, $tag");

	$errors = "";
	if ($shop == "") {
		$errors .= 'Faslet Shop Missing\n';
	}
	if (!$product->meta_exists("Faslet_Brand") || $brand == "") {
		$errors .= 'Faslet Brand Missing\n';
	}
	if (!$product->meta_exists("Faslet_Tag") || $tag == "") {
		$errors .= 'Faslet Tag Missing\n';
	}

	$colors = '[]';
	$output = '[]';
	if ($product instanceof WC_Product_Variable) {

		faslet_log("Variable product");

        $faslet_product_color_attribute = $product->meta_exists("Faslet_Color_Attribute") ? $product->get_meta('Faslet_Color_Attribute') : $FASLET_default_color_attr;
        $faslet_global_color_attribute = get_option("Faslet_Color_Attribute");
        $faslet_color_attribute = $faslet_global_color_attribute ? $faslet_global_color_attribute : $faslet_product_color_attribute;

        $faslet_product_size_attribute = $product->meta_exists("Faslet_Size_Attribute") ? $product->get_meta('Faslet_Size_Attribute') : $FASLET_default_size_attr;
        $faslet_global_size_attribute = get_option("Faslet_Size_Attribute");
        $faslet_size_attribute = $faslet_global_size_attribute ? $faslet_global_size_attribute : $faslet_product_size_attribute;

        $variations = $product->get_available_variations("object");

		$output = '[';
		foreach ($variations as $variation) {
			$var_id = $variation->get_id();
            $var_sku = $variation->get_sku();
            $size = $variation->get_attribute("$faslet_size_attribute");
            $color = $variation->get_attribute("$faslet_color_attribute");
			$available = $variation->is_in_stock() ? 1 : 0;

            $variantPrice = $variation->get_price();

            $variantImage = '';
            @$dom->loadHTML($variation->get_image());
            $imgElt = $dom->getElementsByTagName('img');
            if (count($imgElt) > 0) {
                $variantImage = $imgElt[0]->getAttribute('src');
            }

			$output .= "{ \"size\": \"$size\", \"color\": \"$color\", \"id\": \"$var_id\", \"available\": !!$available, \"sku\": \"$var_sku\", \"price\": \"$variantPrice\", \"imageUrl\": \"$variantImage\" },";
		}
		$output = substr($output, 0, -1);
		$output .= ']';

		faslet_log("Variable product output $output");

		$color_terms = wc_get_product_terms($product_id, $faslet_color_attribute, array('fields' => 'all'));

		$map_to_colors = function ($term) {
			return array('id' => $term->slug, 'name' => $term->name);
		};

		$colors = json_encode(array_map($map_to_colors, $color_terms));

		faslet_log("Color terms $colors");
	}

	$shop_page_url = get_permalink(wc_get_page_id('shop'));

	faslet_log("Shop url $shop_page_url");

    $product_sku = $product->get_sku();

    if($faslet_product_identifier_over_tag == TRUE) {
        $faslet_app_tag = "<faslet-app
            platform=\"woocommerce\"
            product-identifier=\"$faslet_product_id\"
            product-name=\"$name\"
            shop-id=\"$shop\"
            brand=\"$brand\"
            product-img=\"$imageUrl\">
        </faslet-app>";
    } else {
        $faslet_app_tag = "<faslet-app
            platform=\"woocommerce\"
            categories=\"$tag\"
            product-name=\"$name\"
            shop-id=\"$shop\"
            brand=\"$brand\"
            product-img=\"$imageUrl\">
        </faslet-app>";
    }

	print("
		<div class=\"faslet-container\" >
			<script>
				window._faslet = window._faslet || {};
                window._faslet.id = \"$product_id\";
                window._faslet.sku = \"$product_sku\";
				window._faslet.variants = $output;
				window._faslet.shopUrl = \"$shop_page_url\";
				window._faslet.colors = $colors;
				console.log(\"$errors\");
			</script>
			$faslet_app_tag
			<script src=\"https://widget.prod.faslet.net/faslet-app.min.js\" defer></script>
		</div>
	");
}

function faslet_load_script()
{
	global $FASLET_default_hook, $FASLET_debug_enabled;
	$FASLET_debug_enabled = get_option("Faslet_Debug", FALSE);

    if (!class_exists('WooCommerce')) return;

    faslet_log('Load script');

    if (is_product()) {
		faslet_log('Is Product');
		$product_id = get_queried_object_id();
		$product = wc_get_product($product_id);
		faslet_log("Got Product $product_id");

        $faslet_global_hook = get_option("Faslet_Hook");
        $faslet_product_hook = $product->meta_exists("Faslet_Hook") ? $product->get_meta('Faslet_Hook') : $FASLET_default_hook;
        $faslet_hook = $faslet_global_hook ? $faslet_global_hook : $faslet_product_hook;

		faslet_log("Got Position $faslet_hook");
		add_action($faslet_hook, 'faslet_size_me_up', 20);
	}
}

add_action('wp_head', 'faslet_load_script');

function faslet_custom_fields()
{
	global $FASLET_default_size_attr, $FASLET_default_color_attr, $FASLET_default_hook;

	global $post;
	$product = wc_get_product($post->ID);

	$fallback_shop = $product->meta_exists("Faslet_Shop") ?  $product->get_meta('Faslet_Shop') : '';
	$faslet_shop_id = get_option("Faslet_Shop", $fallback_shop);
	$fallback_brand = $product->meta_exists("product_brand") ? $product->get_meta("product_brand") : '';
	$faslet_brand = $product->meta_exists("Faslet_Brand") ? $product->get_meta('Faslet_Brand') : $fallback_brand;
	$faslet_tag = $product->meta_exists("Faslet_Tag") ? $product->get_meta('Faslet_Tag') : '';
	$faslet_product_id = $product->meta_exists("Faslet_ProductId") ? $product->get_meta('Faslet_ProductId') : $product->get_id();
	$faslet_debug = get_option("Faslet_Debug", FALSE);
	$faslet_order_tracking = get_option('Faslet_Order_Tracking', TRUE);
	$faslet_product_identifier_over_tag = get_option('Faslet_Use_ProductId', FALSE);

    $faslet_global_hook = get_option("Faslet_Hook");
    $faslet_product_hook = $product->meta_exists("Faslet_Hook") ? $product->get_meta('Faslet_Hook') : $FASLET_default_hook;
    $faslet_hook = $faslet_global_hook ? $faslet_global_hook : $faslet_product_hook;

	if ($product instanceof WC_Product_Variable) {
        $faslet_product_size_attribute = $product->meta_exists("Faslet_Size_Attribute") ? $product->get_meta('Faslet_Size_Attribute') : $FASLET_default_size_attr;
        $faslet_global_size_attribute = get_option("Faslet_Size_Attribute");
		$faslet_size_attribute = $faslet_global_size_attribute ? $faslet_global_size_attribute : $faslet_product_size_attribute;

        $faslet_product_color_attribute = $product->meta_exists("Faslet_Color_Attribute") ? $product->get_meta('Faslet_Color_Attribute') : $FASLET_default_color_attr;
        $faslet_global_color_attribute = get_option("Faslet_Color_Attribute");
        $faslet_color_attribute = $faslet_global_color_attribute ? $faslet_global_color_attribute : $faslet_product_color_attribute;
	}

	echo '<style>
		.faslet_fields {
			margin: 16px;
		}
		</style>';

	echo '<div class="faslet_fields">';

	echo '<h3>Shared settings</h3>';

	woocommerce_wp_text_input(
		array(
			'id' => 'Faslet_Shop',
			'value' => $faslet_shop_id,
			'placeholder' => 'Faslet Shop Id',
			'label' => __('Faslet Shop Id', 'woocommerce'),
		)
	);

    if ($product instanceof WC_Product_Variable) {
        $taxonomies = wc_get_attribute_taxonomy_labels();

        $attribute_options = array();

        $attribute_options["none"] = "Not Applicable";

        foreach($taxonomies as $name => $label) {
            $attribute_options["pa_$name"] = $label;
        }

        woocommerce_wp_select(
            array(
                'id' => 'Faslet_Size_Attribute',
                'name' => 'Faslet_Size_Attribute',
                'value' => $faslet_size_attribute,
                'label' => __('Faslet Size Attribute', 'woocommerce'),
                'options' => $attribute_options
            )
        );

        woocommerce_wp_select(
            array(
                'id' => 'Faslet_Color_Attribute',
                'name' => 'Faslet_Color_Attribute',
                'value' => $faslet_color_attribute,
                'label' => __('Faslet Color Attribute', 'woocommerce'),
                'options' => $attribute_options
            )
        );
    }

    $options = array(
        "woocommerce_before_add_to_cart_form" =>  __("Before Add to Cart Form", 'woocommerce'),
        "woocommerce_before_variations_form" => __("Before Variations Form", 'woocommerce'),
        "woocommerce_before_add_to_cart_button" => __("Before Add to Cart Button", 'woocommerce'),
        "woocommerce_before_single_variation" => __("Before Single Variation", 'woocommerce'),
        "woocommerce_single_variation" => __("Single Variation", 'woocommerce'),
        "woocommerce_before_add_to_cart_quantity" => __("Before Add to Cart Quantity", 'woocommerce'),
        "woocommerce_after_add_to_cart_quantity" => __("After Add to Cart Quantity", 'woocommerce'),
        "woocommerce_after_single_variation" => __("After Single Variation", 'woocommerce'),
        "woocommerce_after_add_to_cart_button" => __("After Add to Cart Button", 'woocommerce'),
        "woocommerce_after_variations_form" => __("After Variations Form", 'woocommerce'),
        "woocommerce_after_add_to_cart_form" => __("After Add to Cart Form", 'woocommerce'),
        "woocommerce_product_meta_start" => __("Product Meta Start", 'woocommerce'),
        "woocommerce_product_meta_end" => __("Product Meta End", 'woocommerce'),
    );

    woocommerce_wp_select(
        array(
            'id' => 'Faslet_Hook',
            'name' => 'Faslet_Hook',
            'value' => $faslet_hook,
            'label' => __('Faslet Widget Hook', 'woocommerce'),
            'options' => $options
        )
    );

    woocommerce_wp_checkbox(array(
		'id' => 'Faslet_Order_Tracking',
		'name' => 'Faslet_Order_Tracking',
		'value' => $faslet_order_tracking,
		'label' => __('Faslet Order Tracking', 'woocommerce'),
	));

    woocommerce_wp_checkbox(array(
        'id' => 'Faslet_Use_ProductId',
        'name' => 'Faslet_Use_ProductId',
        'value' => $faslet_product_identifier_over_tag,
        'label' => __('Use ProductIdentifier Instead of Tags', 'woocommerce'),
    ));

	woocommerce_wp_checkbox(array(
		'id' => 'Faslet_Debug',
		'name' => 'Faslet_Debug',
		'value' => $faslet_debug,
		'label' => __('Debug Faslet Integration', 'woocommerce'),
	));

	echo '<br><hr/><h3>Product settings</h3>';

	woocommerce_wp_text_input(
		array(
			'id' => 'Faslet_Brand',
			'value' => $faslet_brand,
			'placeholder' => 'Leave blank for auto detect',
			'label' => __('Faslet Brand', 'woocommerce'),
		)
	);

	if($faslet_product_identifier_over_tag)
	{
	    woocommerce_wp_text_input(
    		array(
    			'id' => 'Faslet_ProductId',
    			'value' => $faslet_product_id,
    			'placeholder' => 'Faslet Product Id',
    			'label' => __('Product Identifier', 'woocommerce'),
    		)
    	);
    	echo '<div class="hidden">';
    	woocommerce_wp_text_input(
            array(
                'id' => 'Faslet_Tag',
                'value' => $faslet_tag,
                'placeholder' => 'Leave blank for auto detect',
                'label' => __('Faslet Tag', 'woocommerce'),
            )
        );
        echo '</div>';
	}
	else
	{
	    echo '<div class="hidden">';
	    woocommerce_wp_text_input(
            array(
                'id' => 'Faslet_ProductId',
                'value' => $faslet_product_id,
                'placeholder' => 'Faslet Product Id',
                'label' => __('Product Identifier', 'woocommerce'),
            )
        );
        echo '</div>';
        woocommerce_wp_text_input(
            array(
                'id' => 'Faslet_Tag',
                'value' => $faslet_tag,
                'placeholder' => 'Faslet Tag',
                'label' => __('Faslet Tag', 'woocommerce'),
            )
        );
	}

	echo '<br><hr><h3>Information</h3>';

	echo '<b>Faslet\'s Size Me Up</b> widget for WooCommerce is a paid for service that allows your customers
	to find their perfect fit on the first try.<br>
	<br>In order to activate it for your shop or to get information on the above values or arrange an integration call, please
	<b><a href="mailto:info@faslet.me?subject=Faslet WooCommerce Integration" target="_blank">contact your Faslet Representative</a></b>';

	echo '</div>';
}


function faslet_custom_fields_save($post_id)
{
	$product = wc_get_product($post_id);

	$faslet_debug = isset($_POST['Faslet_Debug']) ? $_POST['Faslet_Debug'] : FALSE;
	update_option('Faslet_Debug', $faslet_debug);

	$faslet_order_tracking = isset($_POST['Faslet_Order_Tracking']) ? $_POST['Faslet_Order_Tracking'] : TRUE;
	update_option('Faslet_Order_Tracking', $faslet_order_tracking);

    $faslet_product_identifier_over_tag = isset($_POST['Faslet_Use_ProductId']) ? $_POST['Faslet_Use_ProductId'] : FALSE;
	update_option('Faslet_Use_ProductId', $faslet_product_identifier_over_tag);

	$faslet_shop_id = isset($_POST['Faslet_Shop']) ? $_POST['Faslet_Shop'] : '';
	update_option('Faslet_Shop', sanitize_text_field($faslet_shop_id));
	update_post_meta($post_id, 'Faslet_Shop', sanitize_text_field($faslet_shop_id));

	$faslet_brand = isset($_POST['Faslet_Brand']) ? $_POST['Faslet_Brand'] : '';
	update_post_meta($post_id, 'Faslet_Brand', sanitize_text_field($faslet_brand));

	$faslet_tag = isset($_POST['Faslet_Tag']) ? $_POST['Faslet_Tag'] : '';
	update_post_meta($post_id, 'Faslet_Tag', sanitize_text_field($faslet_tag));

	$faslet_product_id = isset($_POST['Faslet_ProductId']) ? $_POST['Faslet_ProductId'] : $product->get_id();
    update_post_meta($post_id, 'Faslet_ProductId', sanitize_text_field($faslet_product_id));

	$faslet_hook = isset($_POST['Faslet_Hook']) ? $_POST['Faslet_Hook'] : '';
    update_option('Faslet_Hook', sanitize_text_field($faslet_hook));
	update_post_meta($post_id, 'Faslet_Hook', sanitize_text_field($faslet_hook));

	if ($product instanceof WC_Product_Variable) {
		$faslet_size_attribute = isset($_POST['Faslet_Size_Attribute']) ? $_POST['Faslet_Size_Attribute'] : '';
		update_post_meta($post_id, 'Faslet_Size_Attribute', sanitize_text_field($faslet_size_attribute));
        update_option('Faslet_Size_Attribute', sanitize_text_field($faslet_size_attribute));

		$faslet_color_attribute = isset($_POST['Faslet_Color_Attribute']) ? $_POST['Faslet_Color_Attribute'] : '';
		update_post_meta($post_id, 'Faslet_Color_Attribute', sanitize_text_field($faslet_color_attribute));
        update_option('Faslet_Color_Attribute', sanitize_text_field($faslet_color_attribute));
	}
}

add_action('woocommerce_process_product_meta', 'faslet_custom_fields_save');

function faslet_information_tab($tabs)
{
	print('
		<style>
			#woocommerce-product-data ul.wc-tabs li.faslet_tab_panel a::before {
				font-family: Arial, Sans-Serif;
				content: "F";
				font-weight: 600;
			}
		</style>
	');

	$tabs['faslet_information_tab'] = array(
		'label'  => __('Faslet', 'woocommerce'),
		'target' => 'faslet_tab_panel',
		'class'  => array('faslet_tab_panel'),
	);

	return $tabs;
}

add_filter('woocommerce_product_data_tabs', 'faslet_information_tab');

add_action('woocommerce_product_data_panels', 'faslet_tab_panel');

function faslet_tab_panel()
{
?>
	<div id="faslet_tab_panel" class="panel woocommerce_options_panel">
		<div class="options_group">
			<?php
			faslet_custom_fields()
			?>
		</div>
	</div>
<?php
}

function faslet_order_tracking($order_id)
{
	$faslet_order_tracking_enabled = get_option("Faslet_Order_Tracking", TRUE);
	$shop = get_option("Faslet_Shop", NULL);

	if (is_null($shop)) {
		faslet_log('Shop is null, order tracking disabled');
		return;
	}

	if ($faslet_order_tracking_enabled) {
		$order = wc_get_order($order_id);
		$items = $order->get_items();
		$order_number = json_encode($order->get_order_number());
		$order_status = json_encode($order->get_status());

		$events = "";
		foreach ($items as $item) {
			$name = json_encode($item->get_name());
			$price = json_encode($item->get_total());
			$quantity = json_encode($item->get_quantity());

            $product = is_callable( array( $item, 'get_product' ) ) ? $item->get_product() : false;

            $variant_sku = json_encode($product->get_sku());
            if ($product instanceof WC_Product_Variation) {
                $parent_data = $product->get_parent_data();
                $product_sku = json_encode($parent_data['sku']);
                $product_id = json_encode($product->parent_id);
            } else {
                $product_sku = json_encode($variant_sku);
                $product_id = json_encode($product->id);
            }

            $variant_id = json_encode($product->get_id());

			$events .= "window._faslet_orders.event('widget_order_track','$shop', {
				order: $order_number,
				payment_status: $order_status,
				title: $name,
				variant_id: $variant_id,
				variant: $name,
				price: $price,
				quantity: $quantity,
				sku: $variant_sku,
				correlationId: $product_id,
				variant_sku: $variant_sku,
				product_sku: $product_sku,
			});\n";
		}

		echo "<script src=\"https://widget.prod.faslet.net/faslet-orders.js\"></script>
			<script async src=\"https://www.googletagmanager.com/gtag/js?id=G-6J8TML143D\"></script>
			<script>
				window._faslet_orders.configure();
				$events
			</script>";
	}
}

add_action('woocommerce_thankyou', 'faslet_order_tracking');

?>